@@ -1,4 +1,5 @@
 import logging
+import traceback
 import warnings
 
 try:
@@ -27,7 +28,8 @@ try:
 
 except ImportError:
     # dependency missing, issue a warning
-    logging.error("dependency not found, please install PyQt5 to enable gui")
+    logging.error("dependency not found, please install PyQt6 to enable gui")
+    logging.error(traceback.format_exc())
     import sys
 
     sys.exit()
@@ -47,7 +49,7 @@ import urllib.request
 from functools import partial
 from optparse import OptionParser
 from pathlib import Path
-from shutil import copyfile
+from shutil import copyfile, copytree
 from urllib.parse import parse_qs, urlparse
 
 # Don't remove this line.  The idna encoding
@@ -85,11 +87,11 @@ class CustomWebEnginePage(QWebEnginePage):
             return False
         return True
 
-    def javaScriptConsoleMessage(self, level, msg, line, sourceID):
-        logging.debug(
-            "Javascript console: "
-            + " ; ".join([str(level), str(msg), str(line), str(sourceID)])
-        )
+    # def javaScriptConsoleMessage(self, level, msg, line, sourceID):
+    #    logging.debug(
+    #        "Javascript console: "
+    #        + " ; ".join([str(level), str(msg), str(line), str(sourceID)])
+    #    )
 
 
 class textWindow(QDialog):
@@ -108,11 +110,6 @@ class textWindow(QDialog):
         self.b.resize(400, 200)
 
 
-# DEBUG_PORT = '5588'
-# DEBUG_URL = 'http://127.0.0.1:%s' % DEBUG_PORT
-# os.environ['QTWEBENGINE_REMOTE_DEBUGGING'] = DEBUG_PORT
-
-
 class SelectPluginWindow(QDialog):
     def __init__(self, app, parent=None):
         try:
@@ -145,8 +142,6 @@ class SelectPluginWindow(QDialog):
 
             self.getPlugins()
         except:
-            import traceback
-
             logging.error(traceback.format_exc())
 
     def getPlugins(self):
@@ -191,8 +186,6 @@ class SelectPluginWindow(QDialog):
                 model.appendRow(standardItem)
             self.listView.setModel(model)
         except:
-            import traceback
-
             try:
                 QMessageBox.warning(self, "Error", traceback.format_exc())
             except:
@@ -344,6 +337,20 @@ class MainWindow(QMainWindow):
 
         _version.triggered.connect(trigger)
 
+        if self.app.config["DEBUG_CLI"]:
+            debug = self.bar.addMenu("Debug")
+            _debugpage = QAction(
+                self.style().standardIcon(QStyle.SP_FileDialogContentsView),
+                "Open external debugging",
+                self,
+            )
+            debug.addAction(_debugpage)
+
+            def trigger():
+                QDesktopServices.openUrl(QUrl("http://localhost:5588/"))
+
+            _debugpage.triggered.connect(trigger)
+
         self.showMaximized()
 
     def triggerPlugin(self, pName):
@@ -376,8 +383,6 @@ class MainWindow(QMainWindow):
                         "The new plugins will only be available after restarting TissUUmaps.",
                     )
         except:
-            import traceback
-
             logging.error(traceback.format_exc())
 
 
@@ -628,8 +633,6 @@ class webEngine(QWebEngineView):
                 for path in paths:
                     addRelativePath_aux(state, path, path[0] == "layers")
             except:
-                import traceback
-
                 logging.error(traceback.format_exc())
 
             return state
@@ -669,15 +672,21 @@ class webEngine(QWebEngineView):
             import zipfile
 
             if getattr(sys, "frozen", False):
-                folderPath = sys._MEIPASS
+                zipFolderPath = sys._MEIPASS
             else:
-                folderPath = os.path.dirname(pathlib.Path(__file__))
-            with zipfile.ZipFile(os.path.join(folderPath, "web.zip"), "r") as zip_ref:
+                zipFolderPath = os.path.dirname(pathlib.Path(__file__))
+            with zipfile.ZipFile(
+                os.path.join(zipFolderPath, "web.zip"), "r"
+            ) as zip_ref:
                 zip_ref.extractall(folderpath)
+            for dir in ["css", "js", "misc", "vendor"]:
+                copytree(
+                    os.path.join(zipFolderPath, "static", dir),
+                    os.path.join(folderpath, dir),
+                    dirs_exist_ok=True,
+                )
             # QMessageBox.about(self, "Information", "Export done!")
         except:
-            import traceback
-
             logging.error(traceback.format_exc())
 
     @pyqtSlot(str)
@@ -718,8 +727,6 @@ class webEngine(QWebEngineView):
                 for path in paths:
                     addRelativePath_aux(state, path)
             except:
-                import traceback
-
                 logging.error(traceback.format_exc())
 
             return state
@@ -798,7 +805,7 @@ class webEngine(QWebEngineView):
             return returnDict
         parts = Path(csvpath).parts
         if self.app.basedir != parts[0]:
-            if not self.app.basedir == "C:\mnt\data\shared":
+            if not self.app.basedir == os.path.abspath(self.app.config["SLIDE_DIR"]):
                 QMessageBox.warning(
                     self, "Error", "All files must be in the same drive."
                 )
@@ -838,7 +845,7 @@ class webEngine(QWebEngineView):
             return returnDict
         parts = Path(layerpath).parts
         if self.app.basedir != parts[0]:
-            if not self.app.basedir == "C:\mnt\data\shared":
+            if not self.app.basedir == os.path.abspath(self.app.config["SLIDE_DIR"]):
                 reply = QMessageBox.question(
                     self,
                     "Error",
@@ -857,8 +864,6 @@ class webEngine(QWebEngineView):
         try:
             views._get_slide(imgPath)
         except:
-            import traceback
-
             logging.error(traceback.format_exc())
             QMessageBox.about(
                 self, "Error", "TissUUmaps did not manage to open this image."
@@ -939,8 +944,8 @@ def main():
         metavar="PORT",
         dest="port",
         type="int",
-        default=5000,
-        help="port to listen on [5000]",
+        default=5432,
+        help="port to listen on [5432]",
     )
     parser.add_option(
         "-Q",
@@ -970,6 +975,7 @@ def main():
     (opts, args) = parser.parse_args()
 
     if opts.DEBUG:
+        views.app.config["DEBUG_CLI"] = True
         log = logging.getLogger("werkzeug")
         log.setLevel(logging.DEBUG)
         log = logging.getLogger("pyvips")
@@ -978,8 +984,14 @@ def main():
         log.setLevel(logging.DEBUG)
         warnings.filterwarnings("default")
         logging.debug("Debug mode")
+
+        DEBUG_PORT = "5588"
+        DEBUG_URL = "http://127.0.0.1:%s" % DEBUG_PORT
+        os.environ["QTWEBENGINE_REMOTE_DEBUGGING"] = DEBUG_PORT
+
         # os.environ['WERKZEUG_RUN_MAIN'] = 'true'
     else:
+        views.app.config["DEBUG_CLI"] = False
         log = logging.getLogger("werkzeug")
         log.setLevel(logging.ERROR)
         log = logging.getLogger("pyvips")
@@ -1007,8 +1019,8 @@ def main():
     QtGui.QSurfaceFormat.setDefaultFormat(fmt)
 
     vp = QtGui.QOpenGLVersionProfile(fmt)
-
-    qt_app = QApplication([])
+    os.environ["QTWEBENGINE_CHROMIUM_FLAGS"] = "--no-sandbox --ignore-gpu-blacklist"
+    qt_app = QApplication(sys.argv)
 
     logo = QtGui.QPixmap("static/misc/design/logo.png")
     logo = logo.scaledToWidth(512, Qt.SmoothTransformation)
@@ -1024,7 +1036,7 @@ def main():
 
     qt_app.processEvents()
 
-    port = 5000
+    port = opts.port
     logging.info("Starting port detection")
     while is_port_in_use(port):
         port += 1
