@@ -46,6 +46,7 @@ HTMLElementUtils.inputTypeColor = function (params) {
     var color = document.createElement("input");
     color.setAttribute("type", "color");
     (params.id || null ? color.setAttribute("id", params.id) : null);
+    (params["class"] || null ? color.setAttribute("class", params["class"]) : null);
     var extraAttributes = params.extraAttributes || null;
     if (extraAttributes) {
         for (var attr in extraAttributes) {
@@ -294,9 +295,7 @@ HTMLElementUtils.determinsticHTMLColor = function (key) {
     var U = 0; var V = 0; var y = 128;
 
     if (!isNaN(parseInt(key))) {
-        console.log("b",key);
         key = parseInt(key,10).toString(4).split("").reverse().join("");
-        console.log("a",key);
     }
     
     var maincolor = key.charAt(0).toLowerCase();
