@@ -178,23 +178,28 @@ dataUtils.updateViewOptions = function(data_id, force_reload_all){
             while (len--) { min = +arr[len] < min ? +arr[len] : min; }
             return min;
         }
-        minX = getMin(data_obj["_processeddata"][data_obj["_X"]]);
-        maxX = getMax(data_obj["_processeddata"][data_obj["_X"]]);
-        minY = getMin(data_obj["_processeddata"][data_obj["_Y"]]);
-        maxY = getMax(data_obj["_processeddata"][data_obj["_Y"]]);
-        if (minX <0 || maxX < 500) {
-            let arr = data_obj["_processeddata"][data_obj["_X"]];
-            for (let i = 0; i < arr.length; ++i) {
-                arr[i] = 5000 * (arr[i] - minX) / (maxX - minX);
+        var minX = getMin(data_obj["_processeddata"][data_obj["_X"]]);
+        var maxX = getMax(data_obj["_processeddata"][data_obj["_X"]]);
+        var minY = getMin(data_obj["_processeddata"][data_obj["_Y"]]);
+        var maxY = getMax(data_obj["_processeddata"][data_obj["_Y"]]);
+        if (minX <0 || maxX < 500 || minY <0 || maxY < 500) {
+            var markerTransform;
+            if (maxX - minX > maxY - minY) {
+                markerTransform = 5000 / (maxX - minX);
             }
-            maxX = getMax(arr);
-        }
-        if (minY <0 || maxY < 500) {
-            let arr = data_obj["_processeddata"][data_obj["_Y"]];
-            for (let i = 0; i < arr.length; ++i) {
-                arr[i] = 5000 * (arr[i] - minY) / (maxY - minY);
+            else {
+                markerTransform = 5000 / (maxY - minY);
             }
-            maxY = getMax(arr);
+            let arrX = data_obj["_processeddata"][data_obj["_X"]];
+            for (let i = 0; i < arrX.length; ++i) {
+                arrX[i] = markerTransform * (arrX[i] - minX);
+            }
+            maxX = getMax(arrX);
+            let arrY = data_obj["_processeddata"][data_obj["_Y"]];
+            for (let i = 0; i < arrY.length; ++i) {
+                arrY[i] = markerTransform * (arrY[i] - minY);
+            }
+            maxY = getMax(arrY);
         }
         // We load an empty image at the size of the data.
         if (tmapp["ISS_viewer"].world.getItemCount() > 0) {
@@ -268,6 +273,7 @@ dataUtils.updateViewOptions = function(data_id, force_reload_all){
         interfaceUtils.alert("No size column selected. Impossible to update view.");return;
     }
     data_obj["_scale_factor"]=inputs["scale_factor"].value;
+    data_obj["_coord_factor"]=inputs["coord_factor"].value;
     // Use shape column
     data_obj["_shape_col"]=(radios["shape_col"].checked ? inputs["shape_col"].value : null);
     if (data_obj["_shape_col"]=="null") {
@@ -277,6 +283,16 @@ dataUtils.updateViewOptions = function(data_id, force_reload_all){
     data_obj["_opacity_col"]=(radios["opacity_check"].checked ? inputs["opacity_col"].value : null);
     if (data_obj["_opacity_col"]=="null") {
         interfaceUtils.alert("No opacity column selected. Impossible to update view.");return;
+    }// Use collection column
+    data_obj["_collectionItem_col"]=(radios["collectionItem_col"].checked ? inputs["collectionItem_col"].value : null);
+    data_obj["_collectionItem_fixed"]=(radios["collectionItem_col"].checked ? null : inputs["collectionItem_fixed"].value);
+    if (data_obj["_collectionItem_col"]=="null") {
+        interfaceUtils.alert("No collection item column selected. Impossible to update view.");return;
+    }
+    if (
+        (data_obj["_collectionItem_col"] || data_obj["_collectionItem_fixed"] > 0)
+        && filterUtils._compositeMode != "collection") {
+        //interfaceUtils.alert("Warning, images are not in Collection Mode. Go to \"Image layers > Filter Settings > Merging mode\" to activate Collection Mode.");
     }
     data_obj["_opacity"]=inputs["opacity"].value;
     // Tooltip
@@ -445,7 +461,7 @@ dataUtils.readCSV = function(data_id, thecsv, options) {
                 for (let i = 0; i < row.data.length; ++i) {
                     const value = row.data[i];
                     // Update type flag of column and push value to temporary buffer
-                    rawdata.isnan[i] = rawdata.isnan[i] || isNaN(value);
+                    rawdata.isnan[i] = rawdata.isnan[i] || isNaN(value) || (value == "");
                     rawdata.tmp[i].push(rawdata.isnan[i] ? value : +value);
                 }
                 if (rawdata.tmp[0].length >= 10000) {
