@@ -9,6 +9,7 @@
  * @property {Bool}   filterUtils._filtersUsed - 
  * @property {Object} filterUtils._filters - 
  * @property {Object} filterUtils._filterItems - 
+ * @property {String} filterUtils._compositeMode - 
  */
  filterUtils = {
     // Choose between ["Brightness", "Exposure", "Hue", "Contrast", "Vibrance", "Noise", 
@@ -295,7 +296,8 @@
         id: "filterCompositeMode",
         options:[
             {text:"Channels", value:"source-over"},
-            {text:"Composite", value:"lighter"}
+            {text:"Composite", value:"lighter"},
+            {text:"Collection", value:"collection"}
         ]
     }
     var label = document.createElement("label");
@@ -440,6 +442,45 @@ filterUtils.setCompositeOperation = function(restart) {
     }
     var filterCompositeMode = document.getElementById("filterCompositeMode");
     filterCompositeMode.value = filterUtils._compositeMode;
+    newCompositeMode = filterUtils._compositeMode;
+    if (newCompositeMode == "collection") {
+        tmapp["ISS_viewer"].collectionMode = true;
+        newCompositeMode = "lighter";
+        var collectionLayout = {
+            tileSize: 1, tileMargin: 0.1,
+            columns: Math.ceil(Math.sqrt(tmapp.layers.length))
+        }
+        if (projectUtils._activeState["collectionLayout"] !== undefined) {
+            collectionLayout = projectUtils._activeState["collectionLayout"];
+        }
+        tmapp["ISS_viewer"].world.arrange(collectionLayout);
+        var inputs = document.querySelectorAll(".visible-layers");
+        for(var i = 0; i < inputs.length; i++) {
+            inputs[i].checked = false;
+            inputs[i].click();
+        }
+        tmapp["ISS_viewer"].viewport.goHome();
+        $(".channelRange").hide();
+    }
+    else if (tmapp["ISS_viewer"].collectionMode){
+        tmapp["ISS_viewer"].collectionMode = false;
+        tmapp["ISS_viewer"].world.setAutoRefigureSizes(false);
+        for (var i = 0; i < tmapp["ISS_viewer"].world._items.length; i++) {
+            layer = tmapp.layers[i];
+            var x = layer.x || 0;
+            var y = layer.y || 0;
+            var scale = layer.scale || 1;
+            var  item = tmapp["ISS_viewer"].world._items[i];
+            var layer0X = tmapp[op + "_viewer"].world.getItemAt(0).getContentSize().x;
+            var layerNX = item.getContentSize().x;
+            item.setWidth(scale*layerNX/layer0X);
+            var point = new OpenSeadragon.Point(x/layer0X, y/layer0X);
+            item.setPosition(point);
+        }
+        tmapp["ISS_viewer"].world.setAutoRefigureSizes(true);
+        tmapp["ISS_viewer"].viewport.goHome();
+        $(".channelRange").show();
+    }
     tmapp[op + "_viewer"].compositeOperation = filterUtils._compositeMode;
     for (i = 0; i < tmapp[op + "_viewer"].world.getItemCount(); i++) {
         tmapp[op + "_viewer"].world.getItemAt(i).setCompositeOperation(filterUtils._compositeMode);
