@@ -10,8 +10,6 @@
  */
 glUtils = {
     _initialized: false,
-    _imageSize: [1, 1],
-    _viewportRect: [0, 0, 1, 1],
     _options: {antialias: false, premultipliedAlpha: true, preserveDrawingBuffer: true},
     _markershapes: "misc/markershapes.png",
 
@@ -40,6 +38,7 @@ glUtils = {
     _colorscaleData: {},         // {uid: array of RGBA values, ...}
     _barcodeToLUTIndex: {},      // {uid: dict, ...}
     _barcodeToKey: {},           // {uid: dict, ...}
+    _collectionItemIndex: {},    // {uid: number, ...}
 
     // Global marker settings and info
     _markerScale: 1.0,
@@ -63,10 +62,9 @@ glUtils._markersVS = `
     #define SHAPE_GRID_SIZE 4.0
     #define DISCARD_VERTEX { gl_Position = vec4(2.0, 2.0, 2.0, 0.0); return; }
 
-    uniform vec2 u_imageSize;
-    uniform vec4 u_viewportRect;
     uniform mat2 u_viewportTransform;
     uniform vec2 u_canvasSize;
+    uniform float u_transformIndex;
     uniform float u_markerScale;
     uniform float u_globalMarkerScale;
     uniform vec2 u_markerScalarRange;
@@ -80,12 +78,14 @@ glUtils._markersVS = `
     uniform float u_pickedMarker;
     uniform sampler2D u_colorLUT;
     uniform sampler2D u_colorscale;
+    uniform sampler2D u_transformLUT;
 
     attribute vec4 a_position;
     attribute float a_index;
     attribute float a_scale;
     attribute float a_shape;
     attribute float a_opacity;
+    attribute float a_transform;
     #ifdef USE_INSTANCING
     attribute float a_vertexID;
     #endif  // USE_INSTANCING
@@ -108,9 +108,10 @@ glUtils._markersVS = `
 
     void main()
     {
-        vec2 imagePos = a_position.xy * u_imageSize;
-        vec2 viewportPos = imagePos - u_viewportRect.xy;
-        vec2 ndcPos = (viewportPos / u_viewportRect.zw) * 2.0 - 1.0;
+        float transformIndex = u_transformIndex >= 0.0 ? u_transformIndex : a_transform;
+        vec4 imageTransform = texture2D(u_transformLUT, vec2(transformIndex / 255.0, 0));
+        vec2 viewportPos = a_position.xy * imageTransform.xy + imageTransform.zw;
+        vec2 ndcPos = viewportPos * 2.0 - 1.0;
         ndcPos.y = -ndcPos.y;
         ndcPos = u_viewportTransform * ndcPos;
 
@@ -143,7 +144,7 @@ glUtils._markersVS = `
         }
 
         gl_Position = vec4(ndcPos, 0.0, 1.0);
-        gl_PointSize = a_scale * u_markerScale * u_globalMarkerScale / u_viewportRect.w;
+        gl_PointSize = a_scale * u_markerScale * u_globalMarkerScale;
         gl_PointSize = clamp(gl_PointSize, 2.0, u_maxPointSize);
 
         v_shapeOrigin.x = mod((v_color.a + 0.00001) * 255.0 - 1.0, SHAPE_GRID_SIZE);
@@ -251,11 +252,10 @@ glUtils._pickingVS = `
     #define OP_CLEAR 0
     #define OP_WRITE_INDEX 1
 
-    uniform vec2 u_imageSize;
-    uniform vec4 u_viewportRect;
     uniform mat2 u_viewportTransform;
     uniform vec2 u_canvasSize;
     uniform vec2 u_pickingLocation;
+    uniform float u_transformIndex;
     uniform float u_markerScale;
     uniform float u_globalMarkerScale;
     uniform float u_markerOpacity;
@@ -265,11 +265,13 @@ glUtils._pickingVS = `
     uniform int u_op;
     uniform sampler2D u_colorLUT;
     uniform sampler2D u_shapeAtlas;
+    uniform sampler2D u_transformLUT;
 
     attribute vec4 a_position;
     attribute float a_index;
     attribute float a_scale;
     attribute float a_opacity;
+    attribute float a_transform;
 
     varying vec4 v_color;
 
@@ -282,9 +284,10 @@ glUtils._pickingVS = `
 
     void main()
     {
-        vec2 imagePos = a_position.xy * u_imageSize;
-        vec2 viewportPos = imagePos - u_viewportRect.xy;
-        vec2 ndcPos = (viewportPos / u_viewportRect.zw) * 2.0 - 1.0;
+        float transformIndex = u_transformIndex >= 0.0 ? u_transformIndex : a_transform;
+        vec4 imageTransform = texture2D(u_transformLUT, vec2(transformIndex / 255.0, 0));
+        vec2 viewportPos = a_position.xy * imageTransform.xy + imageTransform.zw;
+        vec2 ndcPos = viewportPos * 2.0 - 1.0;
         ndcPos.y = -ndcPos.y;
         ndcPos = u_viewportTransform * ndcPos;
 
@@ -310,7 +313,7 @@ glUtils._pickingVS = `
 
             vec2 canvasPos = (ndcPos * 0.5 + 0.5) * u_canvasSize;
             canvasPos.y = (u_canvasSize.y - canvasPos.y);  // Y-axis is inverted
-            float pointSize = a_scale * u_markerScale * u_globalMarkerScale / u_viewportRect.w;
+            float pointSize = a_scale * u_markerScale * u_globalMarkerScale;
             pointSize = clamp(pointSize, 2.0, u_maxPointSize);
 
             // Do coarse inside/outside test against bounding box for marker
@@ -379,7 +382,8 @@ glUtils._loadShaderProgram = function(gl, vertSource, fragSource, definitions=""
     gl.bindAttribLocation(program, 2, "a_scale");
     gl.bindAttribLocation(program, 3, "a_shape");
     gl.bindAttribLocation(program, 4, "a_opacity");
-    gl.bindAttribLocation(program, 5, "a_vertexID");
+    gl.bindAttribLocation(program, 5, "a_transform");
+    gl.bindAttribLocation(program, 6, "a_vertexID");
 
     gl.linkProgram(program);
     if (!gl.getProgramParameter(program, gl.LINK_STATUS)) {
@@ -435,8 +439,6 @@ glUtils.loadMarkers = function(uid) {
     const xPosName = dataUtils.data[uid]["_X"];
     const yPosName = dataUtils.data[uid]["_Y"];
     let numPoints = markerData[xPosName].length;
-    const imageWidth = OSDViewerUtils.getImageWidth();
-    const imageHeight = OSDViewerUtils.getImageHeight();
 
     // If new marker data was loaded, we need to assign each barcode an index
     // that we can use with the LUT textures for color, visibility, etc.
@@ -457,6 +459,8 @@ glUtils.loadMarkers = function(uid) {
     const useScaleFromMarker = dataUtils.data[uid]["_scale_col"] != null;
     const markerScaleFactor = dataUtils.data[uid]["_scale_factor"];
     
+    const markerCoordFactor = dataUtils.data[uid]["_coord_factor"];
+    
     const sectorsPropertyName = dataUtils.data[uid]["_pie_col"];
     const usePiechartFromMarker = dataUtils.data[uid]["_pie_col"] != null;
     if (dataUtils.data[uid]["_pie_dict"] && sectorsPropertyName) {
@@ -481,21 +485,28 @@ glUtils.loadMarkers = function(uid) {
 
     const markerOutline = !dataUtils.data[uid]["_no_outline"];
 
-    // Additional info about the vertex format
-    const NUM_COMPONENTS_PER_MARKER = 9;
-    const NUM_BYTES_PER_MARKER = NUM_COMPONENTS_PER_MARKER * 4;
+    const collectionItemPropertyName = dataUtils.data[uid]["_collectionItem_col"];
+    const useCollectionItemFromMarker = dataUtils.data[uid]["_collectionItem_col"] != null;
+    const collectionItemFixed = dataUtils.data[uid]["_collectionItem_fixed"];
+    let collectionItemIndex = collectionItemFixed;
+
+    // Additional info about the vertex format. Make sure you update also
+    // NUM_BYTES_PER_MARKER when making changes to the format!
+    const NUM_BYTES_PER_MARKER = 36;
     const POINT_OFFSET = numPoints * 0,
-          INDEX_OFFSET = numPoints * 4,
-          SCALE_OFFSET = numPoints * 5,
-          SHAPE_OFFSET = numPoints * 6;
-          OPACITY_OFFSET = numPoints * 7;
-          VERTEX_ID_OFFSET = numPoints * 8;
+          INDEX_OFFSET = numPoints * 16,
+          SCALE_OFFSET = numPoints * 20,
+          SHAPE_OFFSET = numPoints * 24;
+          OPACITY_OFFSET = numPoints * 28;
+          TRANSFORM_OFFSET = numPoints * 30;
+          VERTEX_ID_OFFSET = numPoints * 32;
     const POINT_LOCATION = 0,
           INDEX_LOCATION = 1,
           SCALE_LOCATION = 2,
           SHAPE_LOCATION = 3,
           OPACITY_LOCATION = 4;
-          VERTEX_ID_LOCATION = 5;
+          TRANSFORM_LOCATION = 5;
+          VERTEX_ID_LOCATION = 6;
 
     // Extract and upload vertex data for markers. For datasets with tens of of
     // millions of points, the vertex data can be quite large, so we upload the
@@ -511,7 +522,8 @@ glUtils.loadMarkers = function(uid) {
         let bytedata_index = new Float32Array(chunkSize * 1);
         let bytedata_scale = new Float32Array(chunkSize * 1);
         let bytedata_shape = new Float32Array(chunkSize * 1);
-        let bytedata_opacity = new Float32Array(chunkSize * 1);
+        let bytedata_opacity = new Uint16Array(chunkSize * 1);
+        let bytedata_transform = new Uint16Array(chunkSize * 1);
         let bytedata_vertexID = new Uint8Array(chunkSize * 4);
 
         if (usePiechartFromMarker) {
@@ -522,7 +534,8 @@ glUtils.loadMarkers = function(uid) {
             bytedata_index = new Float32Array(chunkSize * numSectors * 1);
             bytedata_scale = new Float32Array(chunkSize * numSectors * 1);
             bytedata_shape = new Float32Array(chunkSize * numSectors * 1);
-            bytedata_opacity = new Float32Array(chunkSize * numSectors * 1);
+            bytedata_opacity = new Uint16Array(chunkSize * numSectors * 1);
+            bytedata_transform = new Uint16Array(chunkSize * numSectors * 1);
             bytedata_vertexID = new Uint8Array(chunkSize * numSectors * 4);
 
             for (let i = 0; i < chunkSize; ++i) {
@@ -530,14 +543,16 @@ glUtils.loadMarkers = function(uid) {
                 const sectors = markerData[sectorsPropertyName][markerIndex].split(";");
                 const piechartAngles = glUtils._createPiechartAngles(sectors);
                 const lutIndex = (keyName != null) ? barcodeToLUTIndex[markerData[keyName][markerIndex]] : 0;
+                const opacity = useOpacityFromMarker ? markerData[opacityPropertyName][markerIndex] : 1.0;
+                if (useCollectionItemFromMarker) collectionItemIndex = markerData[collectionItemPropertyName][markerIndex];
 
                 for (let j = 0; j < numSectors; ++j) {
                     const k = (i * numSectors + j);
                     const sectorIndex = j;
                     hexColor = piechartPalette[j % piechartPalette.length];
 
-                    bytedata_point[4 * k + 0] = markerData[xPosName][markerIndex] / imageWidth;
-                    bytedata_point[4 * k + 1] = markerData[yPosName][markerIndex] / imageHeight;
+                    bytedata_point[4 * k + 0] = markerData[xPosName][markerIndex];
+                    bytedata_point[4 * k + 1] = markerData[yPosName][markerIndex];
                     bytedata_point[4 * k + 2] = lutIndex + sectorIndex * 4096.0;
                     bytedata_point[4 * k + 3] = Number("0x" + hexColor.substring(1,7));
                     bytedata_index[k] = markerIndex;  // Store index needed for picking
@@ -545,7 +560,8 @@ glUtils.loadMarkers = function(uid) {
                     bytedata_shape[k] =
                         Math.floor((j < numSectors - 1 ? piechartAngles[j + 1] : 0.0) * 4095.0) +
                         Math.floor(piechartAngles[j] * 4095.0) * 4096.0;
-                    bytedata_opacity[k] = useOpacityFromMarker ? markerData[opacityPropertyName][markerIndex] : 1.0;
+                    bytedata_opacity[k] = Math.floor(Math.max(0.0, Math.min(1.0, opacity)) * 65535.0);
+                    bytedata_transform[k] = collectionItemIndex;
                     bytedata_vertexID[4 * k + 0] = 0;  // 1st vertex
                     bytedata_vertexID[4 * k + 1] = 1;  // 2nd vertex
                     bytedata_vertexID[4 * k + 2] = 2;  // 3rd vertex
@@ -556,6 +572,8 @@ glUtils.loadMarkers = function(uid) {
             for (let i = 0; i < chunkSize; ++i) {
                 const markerIndex = i + offset;
                 const lutIndex = (keyName != null) ? barcodeToLUTIndex[markerData[keyName][markerIndex]] : 0;
+                const opacity = useOpacityFromMarker ? markerData[opacityPropertyName][markerIndex] : 1.0;
+                if (useCollectionItemFromMarker) collectionItemIndex = markerData[collectionItemPropertyName][markerIndex];
 
                 if (useColorFromMarker) hexColor = markerData[colorPropertyName][i];
                 if (useColorFromColormap) {
@@ -571,14 +589,15 @@ glUtils.loadMarkers = function(uid) {
                     shapeIndex = Math.max(0.0, Math.floor(Number(shapeIndex))) % numShapes;
                 }
 
-                bytedata_point[4 * i + 0] = markerData[xPosName][markerIndex] / imageWidth;
-                bytedata_point[4 * i + 1] = markerData[yPosName][markerIndex] / imageHeight;
+                bytedata_point[4 * i + 0] = markerData[xPosName][markerIndex] * markerCoordFactor;
+                bytedata_point[4 * i + 1] = markerData[yPosName][markerIndex] * markerCoordFactor;
                 bytedata_point[4 * i + 2] = lutIndex + Number(shapeIndex) * 4096.0;
                 bytedata_point[4 * i + 3] = useColorFromColormap ? Number(scalarValue)
                                                                  : Number("0x" + hexColor.substring(1,7));
                 bytedata_index[i] = markerIndex;  // Store index needed for picking
                 bytedata_scale[i] = useScaleFromMarker ? markerData[scalePropertyName][markerIndex] : 1.0;
-                bytedata_opacity[i] = useOpacityFromMarker ? markerData[opacityPropertyName][markerIndex] : 1.0;
+                bytedata_opacity[i] = Math.floor(Math.max(0.0, Math.min(1.0, opacity)) * 65535.0);
+                bytedata_transform[i] = collectionItemIndex;
                 bytedata_vertexID[4 * i + 0] = 0;  // 1st vertex
                 bytedata_vertexID[4 * i + 1] = 1;  // 2nd vertex
                 bytedata_vertexID[4 * i + 2] = 2;  // 3rd vertex
@@ -612,12 +631,13 @@ glUtils.loadMarkers = function(uid) {
             if (newBufferSize != oldBufferSize)
                 gl.bufferData(gl.ARRAY_BUFFER, newBufferSize, gl.STATIC_DRAW);
         }
-        gl.bufferSubData(gl.ARRAY_BUFFER, (POINT_OFFSET + offset * 4) * numSectors * 4, bytedata_point);
-        gl.bufferSubData(gl.ARRAY_BUFFER, (INDEX_OFFSET + offset * 1) * numSectors * 4, bytedata_index);
-        gl.bufferSubData(gl.ARRAY_BUFFER, (SCALE_OFFSET + offset * 1) * numSectors * 4, bytedata_scale);
-        gl.bufferSubData(gl.ARRAY_BUFFER, (SHAPE_OFFSET + offset * 1) * numSectors * 4, bytedata_shape);
-        gl.bufferSubData(gl.ARRAY_BUFFER, (OPACITY_OFFSET + offset * 1) * numSectors * 4, bytedata_opacity);
-        gl.bufferSubData(gl.ARRAY_BUFFER, (VERTEX_ID_OFFSET + offset * 1) * numSectors * 4, bytedata_vertexID);
+        gl.bufferSubData(gl.ARRAY_BUFFER, (POINT_OFFSET + offset * 16) * numSectors, bytedata_point);
+        gl.bufferSubData(gl.ARRAY_BUFFER, (INDEX_OFFSET + offset * 4) * numSectors, bytedata_index);
+        gl.bufferSubData(gl.ARRAY_BUFFER, (SCALE_OFFSET + offset * 4) * numSectors, bytedata_scale);
+        gl.bufferSubData(gl.ARRAY_BUFFER, (SHAPE_OFFSET + offset * 4) * numSectors, bytedata_shape);
+        gl.bufferSubData(gl.ARRAY_BUFFER, (OPACITY_OFFSET + offset * 2) * numSectors, bytedata_opacity);
+        gl.bufferSubData(gl.ARRAY_BUFFER, (TRANSFORM_OFFSET + offset * 2) * numSectors, bytedata_transform);
+        gl.bufferSubData(gl.ARRAY_BUFFER, (VERTEX_ID_OFFSET + offset * 4) * numSectors, bytedata_vertexID);
         gl.bindBuffer(gl.ARRAY_BUFFER, null);
     }
     console.timeEnd("Generate vertex data");
@@ -628,13 +648,15 @@ glUtils.loadMarkers = function(uid) {
     gl.enableVertexAttribArray(POINT_LOCATION);
     gl.vertexAttribPointer(POINT_LOCATION, 4, gl.FLOAT, false, 0, 0);
     gl.enableVertexAttribArray(INDEX_LOCATION);
-    gl.vertexAttribPointer(INDEX_LOCATION, 1, gl.FLOAT, false, 0, numPoints * numSectors * 16);
+    gl.vertexAttribPointer(INDEX_LOCATION, 1, gl.FLOAT, false, 0, INDEX_OFFSET * numSectors);
     gl.enableVertexAttribArray(SCALE_LOCATION);
-    gl.vertexAttribPointer(SCALE_LOCATION, 1, gl.FLOAT, false, 0, numPoints * numSectors * 20);
+    gl.vertexAttribPointer(SCALE_LOCATION, 1, gl.FLOAT, false, 0, SCALE_OFFSET * numSectors);
     gl.enableVertexAttribArray(SHAPE_LOCATION);
-    gl.vertexAttribPointer(SHAPE_LOCATION, 1, gl.FLOAT, false, 0, numPoints * numSectors * 24);
+    gl.vertexAttribPointer(SHAPE_LOCATION, 1, gl.FLOAT, false, 0, SHAPE_OFFSET * numSectors);
     gl.enableVertexAttribArray(OPACITY_LOCATION);
-    gl.vertexAttribPointer(OPACITY_LOCATION, 1, gl.FLOAT, false, 0, numPoints * numSectors * 28);
+    gl.vertexAttribPointer(OPACITY_LOCATION, 1, gl.UNSIGNED_SHORT, true, 0, OPACITY_OFFSET * numSectors);
+    gl.enableVertexAttribArray(TRANSFORM_LOCATION);
+    gl.vertexAttribPointer(TRANSFORM_LOCATION, 1, gl.UNSIGNED_SHORT, false, 0, TRANSFORM_OFFSET * numSectors);
     // Skip enabling vertex ID array, since it will not be used for non-instanced drawing
     gl_ext_vao.bindVertexArrayOES(null);
 
@@ -645,19 +667,22 @@ glUtils.loadMarkers = function(uid) {
     gl.vertexAttribPointer(POINT_LOCATION, 4, gl.FLOAT, false, 0, 0);
     gl_ext_ia.vertexAttribDivisorANGLE(POINT_LOCATION, 1);
     gl.enableVertexAttribArray(INDEX_LOCATION);
-    gl.vertexAttribPointer(INDEX_LOCATION, 1, gl.FLOAT, false, 0, numPoints * numSectors * 16);
+    gl.vertexAttribPointer(INDEX_LOCATION, 1, gl.FLOAT, false, 0, INDEX_OFFSET * numSectors);
     gl_ext_ia.vertexAttribDivisorANGLE(INDEX_LOCATION, 1);
     gl.enableVertexAttribArray(SCALE_LOCATION);
-    gl.vertexAttribPointer(SCALE_LOCATION, 1, gl.FLOAT, false, 0, numPoints * numSectors * 20);
+    gl.vertexAttribPointer(SCALE_LOCATION, 1, gl.FLOAT, false, 0, SCALE_OFFSET * numSectors);
     gl_ext_ia.vertexAttribDivisorANGLE(SCALE_LOCATION, 1);
     gl.enableVertexAttribArray(SHAPE_LOCATION);
-    gl.vertexAttribPointer(SHAPE_LOCATION, 1, gl.FLOAT, false, 0, numPoints * numSectors * 24);
+    gl.vertexAttribPointer(SHAPE_LOCATION, 1, gl.FLOAT, false, 0, SHAPE_OFFSET * numSectors);
     gl_ext_ia.vertexAttribDivisorANGLE(SHAPE_LOCATION, 1);
     gl.enableVertexAttribArray(OPACITY_LOCATION);
-    gl.vertexAttribPointer(OPACITY_LOCATION, 1, gl.FLOAT, false, 0, numPoints * numSectors * 28);
+    gl.vertexAttribPointer(OPACITY_LOCATION, 1, gl.UNSIGNED_SHORT, true, 0, OPACITY_OFFSET * numSectors);
     gl_ext_ia.vertexAttribDivisorANGLE(OPACITY_LOCATION, 1);
+    gl.enableVertexAttribArray(TRANSFORM_LOCATION);
+    gl.vertexAttribPointer(TRANSFORM_LOCATION, 1, gl.UNSIGNED_SHORT, false, 0, TRANSFORM_OFFSET * numSectors);
+    gl_ext_ia.vertexAttribDivisorANGLE(TRANSFORM_LOCATION, 1);
     gl.enableVertexAttribArray(VERTEX_ID_LOCATION);
-    gl.vertexAttribPointer(VERTEX_ID_LOCATION, 1, gl.UNSIGNED_BYTE, false, 0, numPoints * numSectors * 32);
+    gl.vertexAttribPointer(VERTEX_ID_LOCATION, 1, gl.UNSIGNED_BYTE, false, 0, VERTEX_ID_OFFSET * numSectors);
     gl_ext_ia.vertexAttribDivisorANGLE(VERTEX_ID_LOCATION, 0);  // Vertex ID rate must be per-vertex!
     gl_ext_vao.bindVertexArrayOES(null);
 
@@ -675,6 +700,7 @@ glUtils.loadMarkers = function(uid) {
     glUtils._usePiechartFromMarker[uid] = usePiechartFromMarker;
     glUtils._useShapeFromMarker[uid] = useShapeFromMarker;
     glUtils._colorscaleName[uid] = colorscaleName;
+    glUtils._collectionItemIndex[uid] = collectionItemFixed;
     if (useColorFromColormap) {
         glUtils._updateColorScaleTexture(gl, uid, glUtils._textures[uid + "_colorscale"]);
     }
@@ -714,6 +740,7 @@ glUtils.deleteMarkers = function(uid) {
     delete glUtils._colorscaleData[uid];
     delete glUtils._barcodeToLUTIndex[uid];
     delete glUtils._barcodeToKey[uid];
+    delete glUtils._collectionItemIndex[uid];
 
     // Clean up WebGL resources
     gl.deleteBuffer(glUtils._buffers[uid + "_markers"]);
@@ -823,6 +850,52 @@ glUtils.updateColorLUTTextures = function() {
 }
 
 
+glUtils._createTransformLUTTexture = function(gl) {
+    const imageTransforms = new Array(256 * 4);  // TODO
+    const bytedata = new Float32Array(imageTransforms);
+
+    const texture = gl.createTexture();
+    gl.bindTexture(gl.TEXTURE_2D, texture);
+    gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
+    gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);
+    gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.NEAREST);
+    gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.NEAREST);
+    gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, 256, 1, 0, gl.RGBA, gl.FLOAT, bytedata);
+    gl.bindTexture(gl.TEXTURE_2D, null);
+
+    return texture;
+}
+
+
+glUtils._updateTransformLUTTexture = function(texture) {
+    const canvas = document.getElementById("gl_canvas");
+    const gl = canvas.getContext("webgl", glUtils._options);
+
+    // Compute transforms that takes into account if collection mode viewing is
+    // enabled for image layers
+    const imageTransforms = new Array(256 * 4);
+    for (let i = 0; i < tmapp["ISS_viewer"].world.getItemCount(); ++i) {
+        const bounds = tmapp["ISS_viewer"].viewport.getBounds();
+        const image = tmapp["ISS_viewer"].world.getItemAt(i);
+        const imageWidth = image.getContentSize().x;
+        const imageHeight = image.getContentSize().y;
+        const imageBounds = image.getBounds();
+
+        // Compute the scale and shift to be applied to marker positions
+        imageTransforms[i * 4 + 0] = (imageBounds.width / imageWidth) / bounds.width;     // ScaleX
+        imageTransforms[i * 4 + 1] = (imageBounds.height / imageHeight) / bounds.height;  // ScaleY
+        imageTransforms[i * 4 + 2] = -(bounds.x - imageBounds.x) / bounds.width;          // ShiftX
+        imageTransforms[i * 4 + 3] = -(bounds.y - imageBounds.y) / bounds.height;         // ShiftY
+    }
+
+    const bytedata = new Float32Array(imageTransforms);
+
+    gl.bindTexture(gl.TEXTURE_2D, texture);
+    gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, 256, 1, 0, gl.RGBA, gl.FLOAT, bytedata);
+    gl.bindTexture(gl.TEXTURE_2D, null);
+}
+
+
 glUtils._createColorScaleTexture = function(gl) {
     const bytedata = new Uint8Array(256 * 4);
 
@@ -1018,12 +1091,13 @@ glUtils._drawColorPass = function(gl, viewportTransform, markerScaleAdjusted) {
     gl.blendFuncSeparate(gl.SRC_ALPHA, gl.ONE_MINUS_SRC_ALPHA, gl.ONE, gl.ONE_MINUS_SRC_ALPHA);
 
     // Set per-scene uniforms
-    gl.uniform2fv(gl.getUniformLocation(program, "u_imageSize"), glUtils._imageSize);
-    gl.uniform4fv(gl.getUniformLocation(program, "u_viewportRect"), glUtils._viewportRect);
     gl.uniformMatrix2fv(gl.getUniformLocation(program, "u_viewportTransform"), false, viewportTransform);
     gl.uniform2fv(gl.getUniformLocation(program, "u_canvasSize"), [gl.canvas.width, gl.canvas.height]);
     gl.uniform1f(gl.getUniformLocation(program, "u_markerScale"), markerScaleAdjusted);
     gl.uniform1f(gl.getUniformLocation(program, "u_maxPointSize"), glUtils._useInstancing ? 2048 : 256);
+    gl.activeTexture(gl.TEXTURE3);
+    gl.bindTexture(gl.TEXTURE_2D, glUtils._textures["transformLUT"]);
+    gl.uniform1i(gl.getUniformLocation(program, "u_transformLUT"), 3);
     gl.activeTexture(gl.TEXTURE2);
     gl.bindTexture(gl.TEXTURE_2D, glUtils._textures["shapeAtlas"]);
     gl.uniform1i(gl.getUniformLocation(program, "u_shapeAtlas"), 2);
@@ -1033,6 +1107,8 @@ glUtils._drawColorPass = function(gl, viewportTransform, markerScaleAdjusted) {
         gl_ext_vao.bindVertexArrayOES(glUtils._vaos[uid + (glUtils._useInstancing ? "_markers_instanced" : "_markers")]);
 
         // Set per-markerset uniforms
+        gl.uniform1f(gl.getUniformLocation(program, "u_transformIndex"),
+            glUtils._collectionItemIndex[uid] != null ? glUtils._collectionItemIndex[uid] : -1);
         gl.uniform1f(gl.getUniformLocation(program, "u_globalMarkerScale"), glUtils._globalMarkerScale * glUtils._markerScaleFactor[uid]);
         gl.uniform2fv(gl.getUniformLocation(program, "u_markerScalarRange"), glUtils._markerScalarRange[uid]);
         gl.uniform1f(gl.getUniformLocation(program, "u_markerOpacity"), glUtils._markerOpacity[uid]);
@@ -1093,13 +1169,14 @@ glUtils._drawPickingPass = function(gl, viewportTransform, markerScaleAdjusted)
     gl.useProgram(program);
 
     // Set per-scene uniforms
-    gl.uniform2fv(gl.getUniformLocation(program, "u_imageSize"), glUtils._imageSize);
-    gl.uniform4fv(gl.getUniformLocation(program, "u_viewportRect"), glUtils._viewportRect);
     gl.uniformMatrix2fv(gl.getUniformLocation(program, "u_viewportTransform"), false, viewportTransform);
     gl.uniform2fv(gl.getUniformLocation(program, "u_canvasSize"), [gl.canvas.width, gl.canvas.height]);
     gl.uniform2fv(gl.getUniformLocation(program, "u_pickingLocation"), glUtils._pickingLocation);
     gl.uniform1f(gl.getUniformLocation(program, "u_markerScale"), markerScaleAdjusted);
     gl.uniform1f(gl.getUniformLocation(program, "u_maxPointSize"), glUtils._useInstancing ? 2048 : 256);
+    gl.activeTexture(gl.TEXTURE3);
+    gl.bindTexture(gl.TEXTURE_2D, glUtils._textures["transformLUT"]);
+    gl.uniform1i(gl.getUniformLocation(program, "u_transformLUT"), 3);
     gl.activeTexture(gl.TEXTURE2);
     gl.bindTexture(gl.TEXTURE_2D, glUtils._textures["shapeAtlas"]);
     gl.uniform1i(gl.getUniformLocation(program, "u_shapeAtlas"), 2);
@@ -1110,6 +1187,8 @@ glUtils._drawPickingPass = function(gl, viewportTransform, markerScaleAdjusted)
         gl_ext_vao.bindVertexArrayOES(glUtils._vaos[uid + "_markers"]);
 
         // Set per-markerset uniforms
+        gl.uniform1f(gl.getUniformLocation(program, "u_transformIndex"),
+            glUtils._collectionItemIndex[uid] != null ? glUtils._collectionItemIndex[uid] : -1);
         gl.uniform1f(gl.getUniformLocation(program, "u_globalMarkerScale"), glUtils._globalMarkerScale * glUtils._markerScaleFactor[uid]);
         gl.uniform1i(gl.getUniformLocation(program, "u_usePiechartFromMarker"), glUtils._usePiechartFromMarker[uid]);
         gl.uniform1i(gl.getUniformLocation(program, "u_useShapeFromMarker"), glUtils._useShapeFromMarker[uid]);
@@ -1149,13 +1228,12 @@ glUtils.draw = function() {
     const canvas = document.getElementById("gl_canvas");
     const gl = canvas.getContext("webgl", glUtils._options);
 
-    const bounds = tmapp["ISS_viewer"].viewport.getBounds();
-    glUtils._viewportRect = [bounds.x, bounds.y, bounds.width, bounds.height];
-    const homeBounds = tmapp["ISS_viewer"].world.getHomeBounds();
-    glUtils._imageSize = [homeBounds.width, homeBounds.height];
-    const orientationDegrees = tmapp["ISS_viewer"].viewport.getRotation();
+    // Update per-image transforms that take into account if collection mode
+    // viewing is enabled for the image layers
+    glUtils._updateTransformLUTTexture(glUtils._textures["transformLUT"]);
 
-    // The OSD viewer can be rotated, so need to apply the same transform to markers
+    // The OSD viewer can be rotated, so need to also apply rotation to markers
+    const orientationDegrees = tmapp["ISS_viewer"].viewport.getRotation();
     const t = orientationDegrees * (3.141592 / 180.0);
     const viewportTransform = [Math.cos(t), -Math.sin(t), Math.sin(t), Math.cos(t)];
 
@@ -1163,6 +1241,7 @@ glUtils.draw = function() {
     // dependant on screen resolution or window size
     let markerScaleAdjusted = glUtils._markerScale;
     if (glUtils._useMarkerScaleFix) markerScaleAdjusted *= (gl.canvas.height / 900.0);
+    markerScaleAdjusted /= tmapp["ISS_viewer"].viewport.getBounds().height;  // FIXME
 
     gl.clearColor(0.0, 0.0, 0.0, 0.0);
     gl.clear(gl.COLOR_BUFFER_BIT);
@@ -1294,11 +1373,14 @@ glUtils.restoreLostContext = function(event) {
     let canvas = document.getElementById("gl_canvas");
     const gl = canvas.getContext("webgl", glUtils._options);
 
+    gl.getExtension('OES_texture_float');  // Make sure extension is enabled
+
     // Restore shared WebGL objects
     glUtils._programs["markers"] = glUtils._loadShaderProgram(gl, glUtils._markersVS, glUtils._markersFS);
     glUtils._programs["markers_instanced"] = glUtils._loadShaderProgram(gl, glUtils._markersVS, glUtils._markersFS, "#define USE_INSTANCING\n");
     glUtils._programs["picking"] = glUtils._loadShaderProgram(gl, glUtils._pickingVS, glUtils._pickingFS);
     glUtils._textures["shapeAtlas"] = glUtils._loadTextureFromImageURL(gl, glUtils._markershapes);
+    glUtils._textures["transformLUT"] = glUtils._createTransformLUTTexture(gl);
 
     // Restore per-markers WebGL objects
     for (let [uid, numPoints] of Object.entries(glUtils._numPoints)) {
@@ -1330,9 +1412,11 @@ glUtils.init = function() {
 
     const extensions = gl.getSupportedExtensions();
     if (!extensions.includes("OES_vertex_array_object") ||
-        !extensions.includes("ANGLE_instanced_arrays")) {
-        alert("TissUUmaps requires a browser that supports WebGL 1.0 and the following extensions: OES_vertex_array_object, ANGLE_instanced_arrays");
+        !extensions.includes("ANGLE_instanced_arrays") ||
+        !extensions.includes("OES_texture_float")) {
+        alert("TissUUmaps requires a browser that supports WebGL 1.0 and the following extensions: OES_vertex_array_object, ANGLE_instanced_arrays, OES_texture_float");
     }
+    gl.getExtension('OES_texture_float');  // Make sure extension is enabled
 
     // Place marker canvas under the OSD canvas. Doing this also enables proper
     // compositing with the minimap and other OSD elements.
@@ -1343,6 +1427,7 @@ glUtils.init = function() {
     this._programs["markers_instanced"] = this._loadShaderProgram(gl, this._markersVS, this._markersFS, "#define USE_INSTANCING\n");
     this._programs["picking"] = this._loadShaderProgram(gl, this._pickingVS, this._pickingFS);
     this._textures["shapeAtlas"] = this._loadTextureFromImageURL(gl, glUtils._markershapes);
+    this._textures["transformLUT"] = this._createTransformLUTTexture(gl);
 
     this._createColorbarCanvas();  // The colorbar is drawn separately in a 2D-canvas
 
