@@ -91,7 +91,9 @@ projectUtils.getActiveProject = function () {
                 state.layerOpacities[i] = $("#opacity-layer-"+i).val();
                 state.layerVisibilities[i] = $("#visible-layer-"+i).is(":checked");
             });
-            resolve(state);
+            setTimeout(function() {
+                resolve(state);
+            },300);
         }
         makeButtons(callback)
     })
@@ -241,8 +243,8 @@ projectUtils.makeButtonFromTabAux = function (dataset, csvFile, title, comment)
         projectUtils._activeState.markerFiles = [];
     }
     projectUtils._activeState.markerFiles.push(markerFile);
-    markerFile.fromButton = projectUtils._activeState.markerFiles.length;
-    dataUtils.data[dataset].fromButton = projectUtils._activeState.markerFiles.length;
+    markerFile.fromButton = projectUtils._activeState.markerFiles.length - 1;
+    dataUtils.data[dataset].fromButton = projectUtils._activeState.markerFiles.length - 1;
     
     if( Object.prototype.toString.call( markerFile.path ) === '[object Array]' ) {
         interfaceUtils.createDownloadDropdownMarkers(markerFile);
@@ -310,7 +312,9 @@ projectUtils.loadProjectFileFromServer = function(path) {
     }
     */
     document.getElementById("divMarkersDownloadButtons").innerHTML = "";
-
+    if (state.backgroundColor) {
+        $(".openseadragon-canvas")[0].style.backgroundColor=state.backgroundColor;
+    }
     if (state.plugins) {
         state.plugins.forEach(function(pluginName) {
             pluginUtils.addPlugin(pluginName);
@@ -335,6 +339,7 @@ projectUtils.loadProjectFileFromServer = function(path) {
     if (state.markerFiles) {
         state.markerFiles.forEach(function(markerFile, buttonIndex) {
             markerFile["fromButton"] = buttonIndex;
+            // For compatibility reasons:
             if (markerFile.expectedCSV) {
                 projectUtils.convertOldMarkerFile(markerFile);
                 state.hideTabs = true;
@@ -373,7 +378,10 @@ projectUtils.loadProjectFileFromServer = function(path) {
     }
     if (state.menuButtons) {
         state.menuButtons.forEach(function(menuButton, i) {
-            interfaceUtils.addMenuItem([menuButton.text], function(){ window.open(menuButton.url, '_self').focus();});
+            if ( Object.prototype.toString.call( menuButton.text ) !== '[object Array]' ) {
+                menuButton.text = [menuButton.text]
+            }
+            interfaceUtils.addMenuItem(menuButton.text, function(){ window.open(menuButton.url, '_self').focus();});
         });
     }
     if (state.mpp !== undefined) {
@@ -405,14 +413,7 @@ projectUtils.loadProjectFileFromServer = function(path) {
 /**
  * This method is used to load the TissUUmaps layers from state */
  projectUtils.loadLayers = function(state) {
-    tmapp.layers = [];
-    subfolder = window.location.pathname.substring(0, window.location.pathname.lastIndexOf('/'));
-    state.layers.forEach(function(layer) {
-        pathname = window.location.pathname.substring(0, window.location.pathname.lastIndexOf('/') + 1);
-        tmapp.layers.push(
-            {name: layer.name, tileSource: layer.tileSource}
-        )
-    });
+    tmapp.layers = state.layers;
     if (state.filters) {
         filterUtils._filtersUsed = state.filters;
         $(".filterSelection").prop("checked",false);
@@ -425,9 +426,6 @@ projectUtils.loadProjectFileFromServer = function(path) {
     }
     tmapp[tmapp["object_prefix"] + "_viewer"].world.removeAll();
     overlayUtils.addAllLayers();
-    if (state.layerOpacities && state.layerVisibilities) {
-        $(".visible-layers").prop("checked",true);$(".visible-layers").click();
-    }
     if (state.compositeMode) {
         filterUtils._compositeMode = state.compositeMode;
         filterUtils.setCompositeOperation();
@@ -451,6 +449,7 @@ projectUtils.loadProjectFileFromServer = function(path) {
             filterUtils.setCompositeOperation();
         }
         if (state.layerOpacities && state.layerVisibilities) {
+            $(".visible-layers").prop("checked",true);$(".visible-layers").click();
             tmapp.layers.forEach(function(layer, i) {
                 $("#opacity-layer-"+i).val(state.layerOpacities[i]);
                 if (state.layerVisibilities[i] != 0) {
@@ -544,7 +543,7 @@ projectUtils.convertOldMarkerFile = function(markerFile) {
 }
 
 /**
- * Given an array of layers, return the longest common path
+ * @summary Given an array of layers, return the longest common path
  * @param {!Array<!layers>} strs
  * @returns {string}
  */
