@@ -123,7 +123,7 @@ class ImageConverter:
         self.outputImage = outputImage
 
     def convert(self):
-        logging.debug(" ".join("Converting: ", self.inputImage, self.outputImage))
+        logging.debug(" ".join(["Converting: ", self.inputImage, self.outputImage]))
         if not os.path.isfile(self.outputImage):
 
             def convertThread():
@@ -134,11 +134,14 @@ class ImageConverter:
                     if minVal == maxVal:
                         minVal = 0
                         maxVal = 255
-                    if imgVips.percent(1) < 0 or imgVips.percent(99) > 255:
+                    if minVal < 0 or maxVal > 255:
+                        logging.debug(
+                            f"Rescaling image {self.inputImage}: {minVal} - {maxVal} to 0 - 255"
+                        )
                         imgVips = (255.0 * (imgVips - minVal)) / (maxVal - minVal)
                         imgVips = (imgVips < 0).ifthenelse(0, imgVips)
                         imgVips = (imgVips > 255).ifthenelse(255, imgVips)
-                        imgVips = imgVips.scaleimage()
+                    imgVips = imgVips.scaleimage()
                     imgVips.tiffsave(
                         self.outputImage,
                         pyramid=True,
@@ -291,7 +294,7 @@ def _setup():
 def page_not_found(e):
     # note that we set the 404 status explicitly
     # return render_template("tissuumaps.html", isStandalone=app.config["isStandalone"], message="Impossible to load this file", readOnly=app.config["READ_ONLY"])
-    return redirect("/404"), 404, {"Refresh": "1; url=/404"}
+    return redirect("/"), 404, {"Refresh": "1; url=/"}
 
 
 def _get_slide(path, originalPath=None):
@@ -328,8 +331,14 @@ def _get_slide(path, originalPath=None):
 @app.route("/")
 @requires_auth
 def index():
+    indexPath = os.path.abspath(os.path.join(app.basedir, "index.html"))
+    if os.path.isfile(indexPath) and app.config["READ_ONLY"]:
+        directory = os.path.dirname(indexPath)
+        filename = os.path.basename(indexPath)
+        return send_from_directory(directory, filename)
     return render_template(
         "tissuumaps.html",
+        plugins=[p["module"] for p in app.config["PLUGINS"]],
         isStandalone=app.config["isStandalone"],
         readOnly=app.config["READ_ONLY"],
     )
@@ -344,6 +353,12 @@ def base_static(path):
     return send_from_directory(directory, filename)
 
 
+@app.route("/<path:path>.html")
+@requires_auth
+def base_static_redirect(path):
+    return redirect("/web/" + path + ".html")
+
+
 @app.route("/<path:filename>")
 @requires_auth
 def slide(filename):
@@ -351,6 +366,8 @@ def slide(filename):
     if not path:
         path = "./"
     path = os.path.abspath(os.path.join(app.basedir, path, filename))
+    if not os.path.isfile(path) and not os.path.isfile(path + ".dzi"):
+        abort(404)
     # slide = _get_slide(path)
     slide_url = os.path.basename(path) + ".dzi"  # url_for("dzi", path=path)
     jsonProject = {
